@@ -61,22 +61,11 @@ frdp_display_key_press_event (GtkWidget   *widget,
 {
   FrdpDisplay *self = FRDP_DISPLAY (widget);
   FrdpDisplayPrivate *priv = frdp_display_get_instance_private (self);
-  guint16 keycode = key->hardware_keycode;
 
   if (!frdp_display_is_initialized (self))
     return TRUE;
 
-  switch (key->type) {
-    case GDK_KEY_PRESS:
-      frdp_session_send_key (priv->session, FRDP_KEY_EVENT_PRESS, keycode);
-      break;
-    case GDK_KEY_RELEASE:
-      frdp_session_send_key (priv->session, FRDP_KEY_EVENT_RELEASE, keycode);
-      break;
-    default:
-      g_warn_if_reached ();
-      break;
-  }
+  frdp_session_send_key (priv->session, key);
 
   return TRUE;
 }
